﻿<?

################################################################################
## DH_Project_Exporter (RESTORE)                        © André Liebmann 2018 ##
## Version V5.4                                                               ##
##----------------------------------------------------------------------------##
## 2018-05-19                                                                 ##
################################################################################

################################################################################
##                                                                            ##
## Logik (angepasst für IPS5, da ~String entfallen und Links nunmehr case     ##
## sensitive behandelt werden):                                               ##
##                                                                            ##
## 1. Backup unter BackupFolder/BackupName einlesen                           ##
##    (serialisiertes Array aller Objekte nebst Konfiguration des Original-   ##
##    baumes der Management Console erstellt durch Backup -> Endung ~.bak)    ##
##                                                                            ##
## 2. Wenn mehrere Objektbäume im Backup vorhanden werden diese  nacheinander ##
##    erstellt, wobei die Links nur innerhalb des Objektbaumes berücksichtigt ##
##    werden.                                                                 ##
##                                                                            ##
##    Erstellung in Management Console:                                       ##
##    - Kategorien                                                            ##
##    - Instanzen                                                             ##
##    - Variablen nebst Variablenprofilen                                     ##
##    - Skripte                                                               ##
##    - Ereignisse                                                            ##
##    - Media                                                                 ##
##    - Links                                                                 ##
##                                                                            ##
##    Da die Verlinkung nach ObjektIDs erfolgt, sollte $Root = Unterkategorie ##
##    möglich sein. Werden allerdings Objekte, die nicht unter dem Wurzel-    ##
##    verzeichnis ($Root = 0) lagen importiert werden, wie z.B. Instanzen     ##
##    (I/O Instanzen etc.) ist unbedingt $Root = 0 anzugeben !!!              ##
##                                                                            ##
##    Die Verlinkungen erfolgen (im Gegensatz zur Vorversion mit temporärer   ##
##    Link-Anlage und zuletzt überschreiben), wenn alle neu angelegten        ##
##    ObjektIDs vorliegen.                                                    ##
##    Hier wird nach Pufferung der Konfigurationen anhand Vergleich der       ##
##    ObjektID_old <--> ObjektID_new die Verlinkung erst zuletzt erstellt     ##
##                                                                            ##
## 3. Erstellung des WFC-Objektbaumes und dabei Prüfung, ob dieser bereits    ##
##    vorhanden ist. Wenn ja, wird eine Kopie angelegt (problematisch)        ##
##    Wenn eine Kopie auch schon vorhanden, wird diese vorher gelöscht.       ##
##                                                                            ##
## 4. Erstellung importierter Dateien:                                        ##
##                                                                            ##
##    z.B. bei Multimedia unter IP-Symcon/webfront/user/Multimedia            ##
##                              IP-Symcon/media/Multimedia                    ##
##                                                                            ##
################################################################################

################################################################################
##                                                                            ##
## Konfiguration / Anpassen !!!                                               ##
##----------------------------------------------------------------------------##
## 1. Eingabe des Verzeichnisses, wo das Backup abgelegt ist                  ##
## 2. Eingabe des Backup Namens (ggf. mit Datum/Zeit)                         ##
## 3. Stammverzeichnis, wo zu installieren                                    ##
## 4. WFC importieren ?                                                       ##
## 5. Files importieren ?                                                     ##
## 6. Variablenwerte importieren ?                                            ##
## 7. bei Vorhandensein des Verzeichnisses Skriptabbruch ?                    ##
##                                                                            ##
################################################################################

// Eingabe des Verzeichnisses, wo das Backup abgelegt ist (sollte so bleiben) //
$BackupFolder = "D:/IP-Symcon Backup/";

// Eingabe des Backup Namens (ggf. mit Datum/Zeit) /////////////////////////////
$BackupName   = "Backup_Multimedia_2018-05-19";

// Stammverzeichnis, wo zu installieren
$Root  = 0; //ObjectID

// WFC importieren ? ///////////////////////////////////////////////////////////
$RestoreWFC = true; //true or false

// Files importieren ? /////////////////////////////////////////////////////////
$RestoreFiles = true; //true or false

// Variablenwerte importieren ? ////////////////////////////////////////////////
$RestoreVariableValue = true; //true or false

// bei Vorhandensein des Verzeichnisses Skriptabbruch ? ////////////////////////
$Exist = false; //true or false

// max Scriptlaufzeit anpassen /////////////////////////////////////////////////
$max_execution_time = 600; // Default: 30 sec
ini_set("max_execution_time", $max_execution_time);

// max Memory anpassen /////////////////////////////////////////////////////////
$memory_limit = "2048M";
ini_set("memory_limit", $memory_limit);

################################################################################
## Core / ab hier nichts ändern !!!                                           ##
################################################################################

$Kernel = str_replace("\\","/",IPS_GetKernelDir());

// ModuleID des WebFront Configurator (WFC) ////////////////////////////////////
$ModulID = "{3565B1F2-8F7B-4311-A4B6-1BF1D868F39E}";

// WebFront Configurator ID herausfinden (erste) ///////////////////////////////
$WFC_NewID = IPS_GetInstanceListByModuleID($ModulID)[0];

// WebFront Configurator Items und IDs auslesen ////////////////////////////////
$WFC_Existing_Items = WFC_GetItems($WFC_NewID);
//echo("\r\nBereits existente Items: "); print_r($WFC_Existing_Items);
$WFC_Existing_IDs = WFC_Existing_IDs($WFC_Existing_Items);
//echo("\r\nBereits existente IDs: "); print_r($WFC_Existing_IDs);
	
$Structure = file_get_contents($BackupFolder.$BackupName.".bak");

// falls Einlese-Fehler
if($Structure == false)
	{
	echo("Einlesen von ".$BackupFolder.$BackupName.".bak erfolglos -> Abbruch !!!");
	die;
	}
else
	{
	$Log_Backup = "Dateipfad des Backup:\r\n\r\n".$BackupFolder.$BackupName.".bak erfolgreich eingelesen\r\n";
	}
	
$Structure = unserialize($Structure);

//print_r($Structure);

$WFC_OldID                            = $Structure[0]["WFC_ID"];

$ObjectIDs                            = array("OldID" => "NewID", $WFC_OldID => $WFC_NewID);
$CategoryIDs                          = array();
$InstanceIDs                          = array();
$VariableIDs                          = array();
$ScriptIDs                            = array();
$EventIDs                             = array();
$MediaIDs                             = array();
$LinkIDs                              = array();

$Log_SetObject                        = "";
$Log_WFC                              = "";
$Log_Directories                      = "";
$Log_Files                            = "";

// wenn mehrere Objektbäume enthalten diese einzeln auslesen/installieren //////
for($i = 0; $i < count($Structure); ++$i)
	{
	## zur besseren Übersicht Profiles, Scripte und Media in eigenen Arrays ####
	$MC_ObjectTree                        = $Structure[$i]["MC_ObjectTree"];
	$MC_InstanceConfig                    = $Structure[$i]["MC_InstanceConfig"];	
	$MC_CustomVarProfiles                 = $Structure[$i]["MC_CustomVarProfiles"];
	$MC_ScriptContents                    = $Structure[$i]["MC_ScriptContents"];
	$MC_MediaContents                     = $Structure[$i]["MC_MediaContents"];
	
	$WFC_Export_Tree                      = $Structure[$i]["WFC_Export_Tree"];
	$WFC_Export_IDs 	                  = $Structure[$i]["WFC_Export_IDs"];
	$Directories     	                  = $Structure[$i]["Directories"];
	$FileNames   	                      = $Structure[$i]["FileNames"];
	$FileContent    	                  = $Structure[$i]["FileContent"];
	
	// für die Verarbeitung der ObjectIDs (am Schluss) Array anlegen
	$WFC_Export_Trees[]                   = $WFC_Export_Tree;
	
	/*//////////////////////////////////////////////////////////////////////////
	ObjectTree erstellen
	/*//////////////////////////////////////////////////////////////////////////
	
	if(!empty($MC_ObjectTree))
		{	
		SetMC_ObjectTree($MC_ObjectTree, $Root);
		}
	
	/*//////////////////////////////////////////////////////////////////////////
	Webfront Configurator -> Items erstellen
	/*//////////////////////////////////////////////////////////////////////////
	
	if($RestoreWFC == true)
		{
		$Log_WFC .= "\r\nFür Restore ".$i.":\r\n\r\n";
		
		if(!empty($WFC_Export_Tree))
			{	
			//echo("\r\nWFC_Export_IDs: "); print_r($WFC_Export_IDs);
			$WFC_Evaluated_IDs = WFC_Evaluate_IDs($WFC_Export_IDs, $WFC_Existing_IDs);
			//echo("\r\nSich überschneidende IDs: "); print_r($WFC_Evaluated_IDs);
			
			/*//////////////////////////////////////////////////////////////////
			Prüfung ob WFC-Items original oder als Copy vorhanden
			/*//////////////////////////////////////////////////////////////////
			
			// Keines der WFC-Items in [ID] vorhanden //////////////////////////
			if(empty($WFC_Evaluated_IDs['ID']))
				{
				$WFC_Status = 0;
				WFC_Build_Tree($WFC_NewID, $WFC_Export_Tree, $WFC_Evaluated_IDs, $WFC_Status);
				}
				
			// Ein oder mehrere WFC-Items in [ID] vorhanden, aber [ID_Copy] leer
			if(!empty($WFC_Evaluated_IDs['ID']) && empty($WFC_Evaluated_IDs['ID_Copy']))
				{
				$WFC_Status = 1;
				WFC_Build_Tree($WFC_NewID, $WFC_Export_Tree, $WFC_Evaluated_IDs, $WFC_Status);
				}
				
			// [ID] und [ID_Copy] enthalten sich überschneidende WFC-Items /////
			if(!empty($WFC_Evaluated_IDs['ID']) && !empty($WFC_Evaluated_IDs['ID_Copy']))
				{
				// Importierte WFC-Items und [ID_Copy] entsprechen einander
				if(count($WFC_Export_IDs) == count($WFC_Evaluated_IDs['ID_Copy']))
					{
					$WFC_Status = 2;
					WFC_Build_Tree($WFC_NewID, $WFC_Export_Tree, $WFC_Evaluated_IDs, $WFC_Status);
					}
				// Importierte WFC-Items und [ID_Copy] unterschiedlich -> Fehler
				else
					{
					$Log_WFC .= "\r\nFehler aufgetreten -> Kontrolle des WFC vor Import notwendig";	
					}
				}
			}
		else
			{
			$Log_WFC .= "\r\nKeine WFC Items enthalten\r\n\r\n";
			}
		}
	else
		{
		$Log_WFC .= "\r\nEinstellung: keine WFC Items anlegen gewählt !\r\n\r\n";
		}
		
	/*//////////////////////////////////////////////////////////////////////////
	Verzeichnisse erstellen
	/*//////////////////////////////////////////////////////////////////////////
	
	if($RestoreFiles == true)
		{
		$Log_Directories         .= "\r\nFür Restore ".$i.":\r\n\r\n";
		$Log_Files               .= "\r\nFür Restore ".$i.":\r\n\r\n";
		
		if(!empty($Directories))
			{
			foreach($Directories as $Directory)
				{
				//echo($Directory);
				$Kernel = str_replace("\\","/",IPS_GetKernelDir());
				$return = CreateDirectoryPath($Kernel.$Directory);
				
				// Fehlerabfrage
				if($return == true) {$Log_Directories .= "Directory: ".$Kernel.$Directory." existiert bereits...\r\n";}
				else                {$Log_Directories .= "Directory: ".$Kernel.$Directory." angelegt...\r\n";}
				}
			
			/*//////////////////////////////////////////////////////////////////
			Dateien erstellen und .php oder .txt auf ObjectIDs prüfen
			/*//////////////////////////////////////////////////////////////////
			
			foreach($FileNames as $FileName)
				{
				//echo($FileName);
				$FileName = str_replace($Kernel, "", $FileName);
				
				//echo("\r\nFileName: $FileName");
				
				$File = base64_decode($FileContent[$FileName]);
				
				$FileExtension = explode(".", $FileName);
				$FileExtension = end($FileExtension);
				
				if($FileExtension == "php" || $FileExtension == "txt")
					{
					$Log_Files .= "Datei: ".$Kernel.$FileName." Dateiendung ist $FileExtension -> prüfe ObjectIDs...\r\n";	
					$File       = UpdateWebFrontScriptLinkID($File, $ObjectIDs, $FileName);
					}
				else
					{
					$Log_Files .= "Datei: ".$Kernel.$FileName." Dateiendung ist $FileExtension -> keine Prüfung auf ObjectIDs...\r\n";
					}	
				
				// Pfad erstellen
				$Path = $Kernel.$FileName;	
				$datei = fopen($Path,"w");
				$return = fwrite($datei, $File);
				fclose($datei);
				
				// Fehlerabfrage
				if($return == false) {$Log_Files .= "ERROR: Datei ".$Kernel.$FileName." nicht geschrieben...\r\n";}
				else                 {$Log_Files .= "Datei: ".$Kernel.$FileName." geschrieben...\r\n";}
				}
			}
		else
			{
			$Log_Directories .= "\r\nKeine WF Verzeichnisse/Dateien enthalten\r\n\r\n";
			$Log_Files       .= "\r\nKeine WF Dateien zu prüfen\r\n\r\n"; 
			}
		}
	else
		{
		$Log_Directories .= "\r\nEinstellung: keine WF Verzeichnisse/Dateien  anlegen gewählt !\r\n\r\n";
		$Log_Files       .= "\r\nEinstellung: keine WF Dateien anlegen gewählt !\r\n\r\n"; 
		}
	}

/*//////////////////////////////////////////////////////////////////////////////
ObjectIDs aktualisieren / Neuverknüpfung aller eingelesenen Elemente !!!
Dies erfolgt am Ende der Erstellung aller Elemente
/*//////////////////////////////////////////////////////////////////////////////

$Log_UpdateInstanceConnectionID       = "";
$Log_UpdateVariableCustomActionLinkID = "";
$Log_UpdateScriptLinkID               = "";
$Log_UpdateEventLinkID                = "";
$Log_UpdateLinkTargetID               = "";
$Log_UpdateWFCLinkID                  = "";	

// Für Instanzen (neu verknüpfen) //////////////////////////////////////////////
if(!empty($InstanceIDs))
	{
	foreach($InstanceIDs as $InstanceID)
		{
		UpdateInstanceConnectionID($InstanceID, $ObjectIDs, $InstanceConnectBuffer, $InstanceConfigurationBuffer);
		}
	}
else
	{
	$Log_UpdateInstanceConnectionID      .= "Keine Instanzen zum Aktualisieren vorhanden\r\n";	
	}	
	
// Für VariableCustomAction ////////////////////////////////////////////////////
if(!empty($VariableIDs))
	{
	foreach($VariableIDs as $VariableID)
		{
		UpdateVariableCustomActionLinkID($VariableID, $ObjectIDs, $VariableConfigurationBuffer);
		}
	}
else
	{
	$Log_UpdateVariableCustomActionLinkID      .= "Keine CustomAction-ObjectID zum Aktualisieren vorhanden\r\n";	
	}
	
// Für Scripts /////////////////////////////////////////////////////////////////
if(!empty($ScriptIDs))
	{
	foreach($ScriptIDs as $ScriptID)
		{
		UpdateScriptLinkID($ScriptID, $ObjectIDs);
		}
	}
else
	{
	$Log_UpdateScriptLinkID      .= "Keine Scripts zum Aktualisieren vorhanden\r\n";	
	}
	
// Für Events //////////////////////////////////////////////////////////////////
if(!empty($EventIDs))
	{
	foreach($EventIDs as $EventID)
		{
		UpdateEventLinkID($EventID, $ObjectIDs, $EventConfigurationBuffer);
		}
	}
else
	{
	$Log_UpdateEventLinkID       .= "Keine Events zum Aktualisieren vorhanden\r\n";	
	}
	
// Für Links ///////////////////////////////////////////////////////////////////
if(!empty($LinkIDs))
	{
	foreach($LinkIDs as $LinkID)
		{
		UpdateLinkTargetID($LinkID, $ObjectIDs, $LinkConfigurationBuffer);
		}
	}
else
	{
	$Log_UpdateLinkTargetID      .= "Keine Links zum Aktualisieren vorhanden\r\n";		
	}
	
// Für WFC_Items ///////////////////////////////////////////////////////////////
if($RestoreWFC == true)
	{
	if(!empty($WFC_Export_Trees))
		{
		//echo("nicht leer");
		foreach($WFC_Export_Trees as $WFC_Export_Tree)
			{
			if(!empty($WFC_Export_Tree))
				{
				WFC_Read_Tree($WFC_NewID, $WFC_Export_Tree, $ObjectIDs);
				}	
			}
		}
	else
		{
		//echo("leer");
		$Log_UpdateWFCLinkID     .= "Kein WFC-Tree zum Aktualisieren vorhanden\r\n";	
		}
	}	
else
	{
	$Log_UpdateWFCLinkID .= "\r\nEinstellung: keine WFC Items anlegen gewählt !\r\n\r\n";
	}
		
################################################################################
## LOGGING                                                                    ##
################################################################################

$Log = "
################################################################################
Beginn des Logging
################################################################################

Restore: $BackupName\r\n
Datum:\t\t".date("Y-m-d")."
Zeit:\t\t".date("H:i:s")."

Angegebener Pfad in Management Console: ".IPS_GetLocation($Root)."

".$Log_Backup;

$Log .= "
################################################################################
Management Console: Objekte erstellen
################################################################################

".$Log_SetObject;

$Log .= "
################################################################################
Management Console: Aktualisierung der ObjectIDs
################################################################################

Für InstanceConnectionID:\r\n\r\n".$Log_UpdateInstanceConnectionID."

Für Registervariable:\r\n\r\n".$Log_UpdateInstanceConfiguration;

$Log .= "
################################################################################

Für VariableCustomAction:\r\n\r\n".$Log_UpdateVariableCustomActionLinkID;

$Log .= "
################################################################################

Für Skripte:\r\n\r\n".$Log_UpdateScriptLinkID;

$Log .= "
################################################################################

Für Events:\r\n\r\n".$Log_UpdateEventLinkID;

$Log .= "
################################################################################

Für Links:\r\n\r\n".$Log_UpdateLinkTargetID;

$Log .= "
################################################################################	
Webfront Configurator konfigurieren
################################################################################	

WFC-Items erstellen:\r\n\r\n".$Log_WFC;

$Log .= "
################################################################################

WFC-Items auf ObjectIDs prüfen:\r\n\r\n".$Log_UpdateWFCLinkID;

$Log .= "
################################################################################	
Webfront Verzeichnis erstellen
################################################################################	

Verzeichnisse erstellen:\r\n\r\n".$Log_Directories;

$Log .= "
################################################################################

Dateien erstellen und .php oder .txt auf ObjectIDs prüfen:\r\n\r\n".$Log_Files;

$Log .= "
################################################################################

Ergebnis:\r\n\r\n";

// Logging speichern ///////////////////////////////////////////////////////////
$handle = fopen($BackupFolder.$BackupName."_Restore_Logging.txt", "w");
$return = fwrite($handle, $Log);
fclose($handle);

if($return != false) {$Log .= "Logging unter ".$BackupFolder.$BackupName."_Restore_Logging erfolgreich gespeichert...\r\n";}
else                 {$Log .= "ERROR: ".$BackupFolder.$BackupName."_Restore_Logging nicht gespeichert...\r\n";}

$Log .= "
################################################################################
ENDE des Logging
################################################################################
";

// Ausgabe des Logging in Management Console ///////////////////////////////////
echo($Log);

// WFC neu zum Abschluss neu laden /////////////////////////////////////////////
WFC_Reload($WFC_NewID);

################################################################################
## Funktionen                                                                 ##
################################################################################

/*//////////////////////////////////////////////////////////////////////////////
Funktion Objektbaum anlegen
/*//////////////////////////////////////////////////////////////////////////////

function SetMC_ObjectTree($MC_ObjectTree, $ObjectParentID)
	{
	global $Log;
	
	for($i=0;$i<count($MC_ObjectTree);$i++)
		{
		$Object = $MC_ObjectTree[$i];

		$ObjectID = SetObject($Object, $ObjectParentID);

		// Wenn Childs vorhanden
		if($Object["HasChildren"])
			{
			// Childs leeren, sonst Überlagerung
			$Childs = array();
			
			//echo("\r\n\r\nObject: "); print_r($Object["ObjectName"]);
			
			for($child=0;$child<count($Object["ChildrenIDs"]);$child++)
				{
				$Childs[$child]   = $Object["Childs"][$child];
				
				//echo("\r\n    Child: "); print_r($Childs[$child]["ObjectName"]);
				}
			SetMC_ObjectTree($Childs, $ObjectID);
			}	
		}
	return $MC_ObjectTree;
	}

/*//////////////////////////////////////////////////////////////////////////////
Funktion Auslesen der Details nach ObjectType
/*//////////////////////////////////////////////////////////////////////////////

function SetObject($Object, $ObjectParentID)
	{
	global $MC_InstanceConfig;	
	global $MC_CustomVarProfiles;
	global $MC_ScriptContents;
	global $MC_MediaContents;
	global $Exist;

	global $ObjectIDs;
	global $CategoryIDs;
	global $InstanceIDs;
	global $VariableIDs;
	global $ScriptIDs;
	global $EventIDs;
	global $MediaIDs;
	global $LinkIDs;
		
	global $Log_SetObject;
	
	// ObjektType (0: Category, 1: Instance, 2: Variable, 3: Script, 4: Event, 5: Media, 6: Link)
	$ObjectType     = $Object["ObjectType"];
	
	switch($ObjectType)
		{		
		case 0: // Kategorie anlegen
			
			$Category              = $Object;
			$CategoryName          = $Object["ObjectName"];			

			// wenn Schreibschutz für Verzeichnis aktiviert auf Vorhandensein prüfen
			if($Exist == true)
				{
				if(IPS_GetCategoryIDByName($CategoryName, $ObjectParentID))
					{
					$Log_SetObject .= "Kategorie $CategoryName existiert bereits !\r\nSie haben Scriptabbruch bei Vorhandensein gewählt !\r\nAbbruch...";
					//echo($Log_SetObject);
					die;
					}
				}
			
			$ObjectID = CreateCategory ($Category, $ObjectParentID);
			
			$CategoryIDs[] = $ObjectID;
						
			$Log_SetObject .= "Kategorie Name: $CategoryName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
			
		break;
		
		case 1: // Instanz anlegen
		
			$Instance              = $Object; 
			$InstanceName          = $Object["ObjectName"];
			
			// MC_InstanceConfiguration bereitstellen //////////////////////////
			$MC_InstanceConfiguration = $MC_InstanceConfig[$Object["ObjectID"]];
			//print_r($MC_InstanceConfig);
			//print_r($MC_InstanceConfiguration);
			$ObjectID = CreateInstance($Instance, $MC_InstanceConfiguration, $ObjectParentID);
			
			$InstanceIDs[] = $ObjectID;
								
			$Log_SetObject .= "Instanz Name: $InstanceName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
			   
		break;
		
		case 2: // Variable anlegen
		
			$Variable              = $Object;
			$VariableName          = $Object["ObjectName"];
			
			/*//////////////////////////////////////////////////////////////////
			Zuerst enthaltenes Variablenprofil prüfen !!!
			--------------------------------------------------------------------
			Variablenprofil prüfen: System || Custom || leer
			/*//////////////////////////////////////////////////////////////////
			
			if(!empty($Object["ObjectDetails"]["VariableProfile"])) 
				{			
				$VariableProfileName = $Object["ObjectDetails"]["VariableProfile"];
				//echo("$VariableProfileName\r\n");
				}
				
			elseif(!empty($Object["ObjectDetails"]["VariableCustomProfile"])) 
				{			
				$VariableProfileName = $Object["ObjectDetails"]["VariableCustomProfile"];
				//echo("$VariableProfileName\r\n");
				
				// Variablenprofil existiert im Zielsystem -> vergleichen //////
				if(IPS_VariableProfileExists($VariableProfileName))
					{
					//echo("$VariableProfileName prüfen\r\n");
					$VariableProfile_old = IPS_GetVariableProfile($VariableProfileName);
					//print_r($VariableProfile_old );
					$VariableProfile_new = $MC_CustomVarProfiles[$VariableProfileName];		
					//print_r($VariableProfile_new );
					
					// bei Übereinstimmung nur Meldung /////////////////////////
					if($VariableProfile_old === $VariableProfile_new)
						{
						$Log_SetObject .= "Variablenprofil  ".$VariableProfileName." stimmt überein...\r\n";
						}
					// Profil differiert zwar, ist aber ein Systemprofil (~) ///
					elseif($VariableProfileName[0] == "~")
						{
						$Log_SetObject .= "Variablenprofil  ".$VariableProfileName." differiert, ist aber ein Systemprofil...\r\n";
						}
					// kein Systemprofil -> Löschen / neu Erstellen ////////////
					else
						{
						$Log_SetObject .= "Variablenprofil: ".$VariableProfileName." differiert...\r\n";
						
						// Variablenprofil löschen
						IPS_DeleteVariableProfile($VariableProfileName);
						$Log_SetObject .= "Variablenprofil: ".$VariableProfileName." gelöscht...\r\n";
						
						// Variablenprofil erstellen
						$VariableProfile = $MC_CustomVarProfiles[$VariableProfileName];
						CreateVariableProfile($VariableProfileName, $VariableProfile);
						$Log_SetObject .= "Variablenprofil: ".$VariableProfileName." neu angelegt...\r\n";
						}
					}
					
				// Variablenprofil existiert nicht im Zielsystem ///////////////
				if(!IPS_VariableProfileExists($VariableProfileName))
					{
					$VariableProfile = $MC_CustomVarProfiles[$VariableProfileName];
					
					/*//////////////////////////////////////////////////////////
					Wichtig bei Umstellung auf IPS 5:
					zu importierendes Variablenprofil war ein Systemprofil (z.B.
					~String) so ist es neu anzulegen, wobei das Sonderzeichen
					entfernt werden muss
					/*//////////////////////////////////////////////////////////
					
					// erstes Zeichen auf "~" prüfen und ggf. entfernen					
					if ($VariableProfileName[0] == "~") 
						{ 
						$VariableProfileName_new = substr($VariableProfileName,1,strlen($VariableProfileName)-1); 
						
						// auf angepassten VariableProfileName prüfen
						if(!IPS_VariableProfileExists($VariableProfileName_new))
							{ 
							$VariableProfileName = $VariableProfileName_new;
							
							CreateVariableProfile($VariableProfileName, $VariableProfile);
							$Log_SetObject .= "Variablenprofil: ".$VariableProfileName." neu angelegt...\r\n";
							}
						}
					else
						{					
						CreateVariableProfile($VariableProfileName, $VariableProfile);
						$Log_SetObject .= "Variablenprofil: ".$VariableProfileName." neu angelegt...\r\n";
						}
					}
				}
			else
				{
				$VariableProfileName = "";
				}	
		
			$ObjectID = CreateVariable ($Variable, $VariableProfileName, $ObjectParentID);
			
			// Variablenwerte schreiben, wenn vorhanden ////////////////////////
			if(!empty($Object["ObjectDetails"]["Value"]))
				{
				SetValue($ObjectID, $Object["ObjectDetails"]["Value"]);
				}
				
			$VariableIDs[] = $ObjectID;
						
			$Log_SetObject .= "Variable Name: $VariableName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
						
		break;
		
		case 3: // Skript anlegen
			
			$Script                = $Object;
			$ScriptName            = $Object["ObjectName"];
			
			// Skriptinhalt bereitstellen //////////////////////////////////////
			$ScriptContent         = $MC_ScriptContents[$Object["ObjectID"]];
						
			$ObjectID = CreateScript ($Script, $ScriptContent, $ObjectParentID);
			
			$ScriptIDs[] = $ObjectID;
			
			$Log_SetObject .= "Script Name: $ScriptName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
			
		break;
		
		case 4: // Ereignis anlegen
			
			$Event                 = $Object;
			$EventName             = $Object["ObjectName"];
		
			$ObjectID = CreateEvent($Event, $ObjectParentID);
			
			$EventIDs[] = $ObjectID;
						
			$Log_SetObject .= "Event Name: $EventName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
			
		break;
		
		case 5: // Media anlegen		
			
			$Media                 = $Object;
			$MediaName             = $Object["ObjectName"];
			
			// Medieninhalt bereitstellen //////////////////////////////////////
			$MediaContent          = $MC_MediaContents[$Object["ObjectID"]];

			$ObjectID = CreateMedia($Media, $MediaContent, $ObjectParentID);
			
			$MediaIDs[] = $ObjectID;
						
			$Log_SetObject .= "Media Name: $MediaName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
			
		break;
		
		case 6: // Link anlegen	

			$Link                  = $Object;
			$LinkName              = $Object["ObjectName"];
			
			$ObjectID = CreateLink($Link, $ObjectParentID);
			
			$LinkIDs[] = $ObjectID;
						
			$Log_SetObject .= "Link Name: $LinkName | ID: $ObjectID | ParentID: $ObjectParentID wurde angelegt...\r\n";
			
		break;
		}
	
	// Array mit alten und neuen ObjectIDs //////////////////////////////////////	
	$ObjectID_old = $Object["ObjectID"];
	$ObjectID_new = $ObjectID;
	$ObjectIDs[$ObjectID_old] = $ObjectID_new;
		
	return $ObjectID;
	}
		
/*//////////////////////////////////////////////////////////////////////////////
Anlegen einer Kategorie:
/*//////////////////////////////////////////////////////////////////////////////

function CreateCategory($Category, $ParentID)
	{
	$CategoryID = IPS_CreateCategory();                        // anlegen
	IPS_SetName($CategoryID, $Category["ObjectName"]);         // benennen
	IPS_SetParent($CategoryID, $ParentID);                     // einsortieren unter $ObjectParentID
	IPS_SetPosition($CategoryID, $Category["ObjectPosition"]); // Position festlegen
	
	IPS_SetInfo($CategoryID, $Category["ObjectInfo"]);
	IPS_SetIcon($CategoryID, $Category["ObjectIcon"]);
	IPS_SetHidden($CategoryID, $Category["ObjectIsHidden"]);
	
	return $CategoryID;
	}
						 
/*//////////////////////////////////////////////////////////////////////////////
Anlegen einer Instanz:
/*//////////////////////////////////////////////////////////////////////////////

function CreateInstance($Instance, $MC_InstanceConfiguration, $ParentID)
	{ 
	global $InstanceConnectBuffer;
	global $InstanceConfigurationBuffer;
	
	// anzulegende Instanz
	$InstanceType = $Instance["ObjectDetails"]["ModuleInfo"]["ModuleID"];
	$InstanceConnectionID = $Instance["ObjectDetails"]["ConnectionID"];	
	$InstanceID = IPS_CreateInstance($InstanceType);              // erstellen von Typ
	IPS_SetName($InstanceID, $Instance["ObjectName"]);            // benennen
	IPS_SetParent($InstanceID, $ParentID);                        // einsortieren unter Kategorie
	IPS_SetPosition($InstanceID, $Instance["ObjectPosition"]);    // Position festlegen
	
	IPS_SetInfo($InstanceID, $Instance["ObjectInfo"]);
	IPS_SetIcon($InstanceID, $Instance["ObjectIcon"]);
	IPS_SetHidden($InstanceID, $Instance["ObjectIsHidden"]);

	// noch keine Konfiguration festlegen, sondern erst, wenn ObjectIDs geprüft	
	
	// Buffer erstellen für ConnectInstance
	$InstanceConnectBuffer[$InstanceID] = $InstanceConnectionID;
	// Buffer erstellen für InstanceConfiguration
	$InstanceConfigurationBuffer[$InstanceID] = $MC_InstanceConfiguration;

	return $InstanceID;
	}

/*//////////////////////////////////////////////////////////////////////////////
Anlegen einer Variablen
--------------------------------------------------------------------------------
$VariableType:
0 = Boolean | 1 = Integer | 2 = Float | 3 = String
/*//////////////////////////////////////////////////////////////////////////////

function CreateVariable ($Variable, $VariableProfileName, $ParentID)
	{
	global $VariableConfigurationBuffer;
	
	$VariableConfiguration = $Variable["ObjectDetails"];
	$VariableType          = $Variable["ObjectDetails"]["VariableType"];
				
	$VariableID = IPS_CreateVariable($VariableType);                 // erstellen von Typ
	IPS_SetName($VariableID, $Variable["ObjectName"]);               // benennen
	IPS_SetParent($VariableID, $ParentID);                           // einsortieren unter Kategorie
	IPS_SetPosition($VariableID, $Variable["ObjectPosition"]);       // Position festlegen
	
	IPS_SetInfo($VariableID, $Variable["ObjectInfo"]);
	IPS_SetIcon($VariableID, $Variable["ObjectIcon"]);
	IPS_SetHidden($VariableID, $Variable["ObjectIsHidden"]);

	IPS_SetVariableCustomProfile($VariableID, $VariableProfileName); // Profil festlegen
	
	// Buffer erstellen für InstanceConfiguratione
	$VariableConfigurationBuffer[$VariableID] = $VariableConfiguration;
	
	return $VariableID;
	}

/*//////////////////////////////////////////////////////////////////////////////
Anlegen eines Variablenprofils
/*//////////////////////////////////////////////////////////////////////////////

function CreateVariableProfile($VariableProfileName, $VariableProfile)
	{
	IPS_CreateVariableProfile($VariableProfileName, $VariableProfile["ProfileType"]);
	IPS_SetVariableProfileIcon($VariableProfileName, $VariableProfile["Icon"]);
	IPS_SetVariableProfileText($VariableProfileName, $VariableProfile["Prefix"], $VariableProfile["Suffix"]);
	
	IPS_SetVariableProfileValues($VariableProfileName, $VariableProfile["MinValue"], $VariableProfile["MaxValue"], $VariableProfile["StepSize"]);
	IPS_SetVariableProfileDigits($VariableProfileName, $VariableProfile["Digits"]);
	
	if(isset($VariableProfile["Associations"]))
		{
		foreach($VariableProfile["Associations"] as $Association)
			{
			IPS_SetVariableProfileAssociation($VariableProfileName, $Association["Value"], $Association["Name"], $Association["Icon"], $Association["Color"]);
			}
		}
	}

/*//////////////////////////////////////////////////////////////////////////////
Anlegen eines Skriptes
--------------------------------------------------------------------------------
$ScriptType 0 erzeugt ein normales PHP Skript
/*//////////////////////////////////////////////////////////////////////////////

function CreateScript($Script, $ScriptContent, $ParentID)
	{
	$ScriptConfiguration = $Script["ObjectDetails"];
				
	$ScriptID = IPS_CreateScript($ScriptConfiguration["ScriptType"]); // anlegen
	IPS_SetName($ScriptID, $Script["ObjectName"]);                    // benennen
	IPS_SetParent($ScriptID, $ParentID);                              // einsortieren unter Kategorie
	IPS_SetPosition($ScriptID, $Script["ObjectPosition"]);            // Position festlegen
	
	//wenn eigener Scriptname vorhanden (z.B. "Example.ips.php") statt ObjectID.ips.php
	$ScriptName = $ScriptConfiguration["ScriptFile"];
	$Name = explode(".", $ScriptName);
	
	if(!is_numeric($Name[0]))
		{ 
		$Kernel = str_replace("\\","/",IPS_GetKernelDir());
		//Datei umbenennen in $ScriptName
		rename($Kernel.'scripts/'.$ScriptID.'.ips.php', $Kernel.'scripts/'.$ScriptName);
		//Dem Skript den neuen Dateinamen ("Example.ips.php") zuweisen
		IPS_SetScriptFile($ScriptID, $ScriptName);
		//echo("\r\nScriptID: $ScriptID mit geänderten ScriptName: $ScriptName geschrieben ...\r\n");	
		}	
		
	IPS_SetInfo($ScriptID, $Script["ObjectInfo"]);
	IPS_SetIcon($ScriptID, $Script["ObjectIcon"]);
	IPS_SetHidden($ScriptID, $Script["ObjectIsHidden"]);
	
	IPS_SetScriptContent($ScriptID, $ScriptContent);                  // Scriptinhalt festlegen
	
	return $ScriptID;
	}

/*//////////////////////////////////////////////////////////////////////////////
Event-Erstellung
--------------------------------------------------------------------------------
zu übergeben:
$Event als Object-Beschreibung mit Array ObjectDetails (IPS_GetConfiguration)
--------------------------------------------------------------------------------
$EventType:
0 ausgelöst | 1 zyklisch | 2 Wochenplan
/*//////////////////////////////////////////////////////////////////////////////

function CreateEvent($Event, $ParentID)
	{
	global $EventConfigurationBuffer;
	
	$EventConfiguration = $Event["ObjectDetails"];
	
	$EventID = IPS_CreateEvent($EventConfiguration["EventType"]);
	IPS_SetName($EventID, $Event["ObjectName"]);
	IPS_SetParent($EventID, $ParentID);
	IPS_SetPosition($EventID, $Event["ObjectPosition"]);
	
	IPS_SetInfo($EventID, $Event["ObjectInfo"]);
	IPS_SetIcon($EventID, $Event["ObjectIcon"]);
	IPS_SetHidden($EventID, $Event["ObjectIsHidden"]);
	
	if($EventConfiguration["EventType"] == 0) // Ausgelöstes Ereignis
		{
		if($EventConfiguration["TriggerValue"] != 0)               {IPS_SetEventTriggerValue($EventID, $EventConfiguration["TriggerValue"]);}
		if($EventConfiguration["TriggerSubsequentExecution"] != 0) {IPS_SetEventTriggerSubsequentExecution($EventID, $EventConfiguration["TriggerSubsequentExecution"]);}
		}
	if($EventConfiguration["EventType"] == 1) // Zyklisches Ereignis
		{
		IPS_SetEventCyclic($EventID, $EventConfiguration["CyclicDateType"], $EventConfiguration["CyclicDateValue"], $EventConfiguration["CyclicDateDay"], $EventConfiguration["CyclicDateDayValue"], $EventConfiguration["CyclicTimeType"], $EventConfiguration["CyclicTimeValue"]);	
		IPS_SetEventCyclicDateFrom($EventID, $EventConfiguration["CyclicDateFrom"]["Day"], $EventConfiguration["CyclicDateFrom"]["Month"], $EventConfiguration["CyclicDateFrom"]["Year"]);
		IPS_SetEventCyclicDateTo($EventID, $EventConfiguration["CyclicDateTo"]["Day"], $EventConfiguration["CyclicDateTo"]["Month"], $EventConfiguration["CyclicDateTo"]["Year"]);	
		IPS_SetEventCyclicTimeFrom($EventID, $EventConfiguration["CyclicTimeFrom"]["Hour"], $EventConfiguration["CyclicTimeFrom"]["Minute"], $EventConfiguration["CyclicTimeFrom"]["Second"]);
		IPS_SetEventCyclicTimeTo($EventID, $EventConfiguration["CyclicTimeTo"]["Hour"], $EventConfiguration["CyclicTimeTo"]["Minute"], $EventConfiguration["CyclicTimeTo"]["Second"]);
		}
	if($EventConfiguration["EventType"] == 2) // Wochenplan
		{
		foreach($EventConfiguration["ScheduleActions"] as $EventScheduleAction)
			{
			IPS_SetEventScheduleAction($EventID, $EventScheduleAction["ID"], $EventScheduleAction["Name"],  $EventScheduleAction["Color"], $EventScheduleAction["ScriptText"]); 
			}
			
		foreach($EventConfiguration["ScheduleGroups"] as $EventScheduleGroup)
			{
			IPS_SetEventScheduleGroup($EventID, $EventScheduleGroup["ID"], $EventScheduleGroup["Days"]);
			
			for($i=0;$i<count($EventScheduleGroup["Points"]);$i++)
				{
				IPS_SetEventScheduleGroupPoint($EventID, $EventScheduleGroup["ID"], $i, $EventScheduleGroup["Points"][$i]["Start"]["Hour"], $EventScheduleGroup["Points"][$i]["Start"]["Minute"], $EventScheduleGroup["Points"][$i]["Start"]["Second"], $EventScheduleGroup["Points"][$i]["ActionID"]);   
				}
			}
		}
	
	// Ereignis aktiviert/inaktiviert
	IPS_SetEventActive ($EventID, $EventConfiguration["EventActive"]);	
	// limitiert die Anzahl der Ereignisaufrufe
	IPS_SetEventLimit ($EventID, $EventConfiguration["EventLimit"]);

	// Buffer erstellen für EventConfiguration
	$EventConfigurationBuffer[$EventID] = $EventConfiguration;
		
	return $EventID;
	}

/*//////////////////////////////////////////////////////////////////////////////
Anlegen eines Mediaobjektes
--------------------------------------------------------------------------------
$MediaType:
0 = Designer | 1 = Image | 2 = Sound | 3 = Stream | 4	Chart | 5 = Dokument
/*//////////////////////////////////////////////////////////////////////////////

function CreateMedia($Media, $MediaContent, $ParentID)
	{
	$MediaConfiguration = $Media["ObjectDetails"];
	
	$MediaID = IPS_CreateMedia($MediaConfiguration["MediaType"]);        // anlegen
	IPS_SetName($MediaID, $Media["ObjectName"]);                         // benennen
	IPS_SetParent($MediaID, $ParentID);                                  // einsortieren unter Kategorie
	IPS_SetPosition($MediaID, $Media["ObjectPosition"]);                 // Position festlegen 
		   	
	IPS_SetInfo($MediaID, $Media["ObjectInfo"]);
	IPS_SetIcon($MediaID, $Media["ObjectIcon"]);
	IPS_SetHidden($MediaID, $Media["ObjectIsHidden"]);
	   
	IPS_SetMediaFile($MediaID, $MediaConfiguration["MediaFile"], false); // Image im MedienPool mit Image-Datei verbinden
	IPS_SetMediaContent($MediaID, $MediaContent);                        // Datei befüllen
	
	return $MediaID;
	}

/*//////////////////////////////////////////////////////////////////////////////
Anlegen eines Link
/*//////////////////////////////////////////////////////////////////////////////

function CreateLink($Link, $ParentID)
	{
	global $LinkConfigurationBuffer;
				
	$LinkID = IPS_CreateLink();                                   // anlegen
	IPS_SetName($LinkID, $Link["ObjectName"]);                    // benennen
	IPS_SetParent($LinkID, $ParentID);                            // einsortieren unter Kategorie
	IPS_SetPosition($LinkID, $Link["ObjectPosition"]);            // Position festlegen
		
	IPS_SetInfo($LinkID, $Link["ObjectInfo"]);
	IPS_SetIcon($LinkID, $Link["ObjectIcon"]);
	IPS_SetHidden($LinkID, $Link["ObjectIsHidden"]);
	
	// Buffer erstellen für LinkConfiguration
	$LinkConfigurationBuffer[$LinkID] = $Link["ObjectDetails"];
	
	return $LinkID;
	}

/*//////////////////////////////////////////////////////////////////////////////
Update der in Instanzen enthaltenen ObjectIDs --> zuerst übergeordnete Instanzen
dann Prüfung, ob eine RXObjectID enthalten (bei Registervariablen)
erst dann wird die Konfiguration der Instanz schreiben
/*//////////////////////////////////////////////////////////////////////////////

function UpdateInstanceConnectionID($InstanceID, $ObjectIDs, $InstanceConnectBuffer, $InstanceConfigurationBuffer)
	{
	global $Log_UpdateInstanceConnectionID;
	global $Log_UpdateInstanceConfiguration;

	$InstanceConnectionID = $InstanceConnectBuffer[$InstanceID];

	if(($InstanceConnectionID != ""))
		{
		$OldID = $InstanceConnectionID;
		$NewID = $ObjectIDs[$OldID];
		IPS_ConnectInstance($InstanceID, $NewID);
		IPS_ApplyChanges($InstanceID);
		$Log_UpdateInstanceConnectionID .= ("Instanz $InstanceID enthält übergeordnete Instanz OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
		}
	else
		{
		$Log_UpdateInstanceConnectionID .= ("Instanz $InstanceID enthält keinen Link zu einer übergeordneten Instanz\r\n");
		}	

	// $InstanceConfigurationBuffer auf lokale IP (bei Sockets) prüfen
	$Configuration = json_decode($InstanceConfigurationBuffer[$InstanceID]); //print_r($Configuration);
	
	// wenn exportierte lokale IP enthalten --> löschen
	if(isset($Configuration->BindIP))
		{
		unset($Configuration->BindIP);
		
		$Log_UpdateInstanceConfiguration .= ("Instanz $InstanceID enthält  exportiertelokale IP -> gelöscht\r\n");
		}
	else
		{
		$Log_UpdateInstanceConfiguration .= ("Instanz $InstanceID enthält keine exportierte lokale IP\r\n");
		}	
	
	// wenn RXObjectID enthalten --> verknüpfte ObjectID ersetzen
	if(isset($Configuration->RXObjectID))
		{
		$OldID = $Configuration->RXObjectID;
		$NewID = $ObjectIDs[$OldID];
		$Configuration->RXObjectID = $NewID;
		
		$Log_UpdateInstanceConfiguration .= ("Instanz $InstanceID enthält RXObjectID: $OldID -> ersetze durch NewID: $NewID\r\n");
		}
	else
		{
		$Log_UpdateInstanceConfiguration .= ("Instanz $InstanceID enthält keine RXObjectID\r\n");
		}	
	
	$InstanceConfiguration = json_encode($Configuration);
							
	IPS_SetConfiguration($InstanceID, $InstanceConfiguration); // Konfiguration festlegen
	IPS_ApplyChanges($InstanceID);
	}
		
/*//////////////////////////////////////////////////////////////////////////////
Update der in Scripten enthaltenen LinkID`s
/*//////////////////////////////////////////////////////////////////////////////

function UpdateVariableCustomActionLinkID($VariableID, $ObjectIDs, $VariableConfigurationBuffer)
	{
	global $Log_UpdateVariableCustomActionLinkID;

	$VariableCustomActionLinkID = $VariableConfigurationBuffer[$VariableID]["VariableCustomAction"];
	//echo($VariableCustomActionLinkID);
	
	//neuen Variable durchsuchen nach VariableCustomAction und ersetzen/////////
	if(($VariableCustomActionLinkID != "") && ($VariableCustomActionLinkID != "0"))
		{
		$OldID = $VariableCustomActionLinkID;
		$NewID = $ObjectIDs[$OldID];
		IPS_SetVariableCustomAction($VariableID, $NewID);
		$Log_UpdateVariableCustomActionLinkID .= ("Variable $VariableID enthält CustomAction-ObjectID OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
		}
	else
		{
		$Log_UpdateVariableCustomActionLinkID .= ("Variable $VariableID enthält keinen Link zu einer CustomAction-ObjectID\r\n");
		}
	}
	
/*//////////////////////////////////////////////////////////////////////////////
Update der in Scripten enthaltenen LinkID`s
/*//////////////////////////////////////////////////////////////////////////////

function UpdateScriptLinkID($ScriptID, $ObjectIDs)
	{
	$ScriptContent = IPS_GetScriptContent($ScriptID);
	
	global $Log_UpdateScriptLinkID;
	
	/*//////////////////////////////////////////////////////////////////////////
	suche 5stellige Zahlen in Dateien mit Endung .php | .txt
	----------------------------------------------------------------------------
	Treffer nur wenn
	- davor: keine Zahl \d | kein : | kein Groß- oder Kleinbuchstabe
	- 5 Zahlen
	- danach: keine Zahl 
	/*//////////////////////////////////////////////////////////////////////////

	preg_match_all ("/(?<![\d\:A-Za-z])([1-5][0-9]{4})(?!\d)/", $ScriptContent, $OldIDs); 
	
	if(!empty($OldIDs[0]))
		{
		foreach($OldIDs[0] as $OldID)
			{
			if(isset($ObjectIDs[$OldID]))
				{
				$NewID = $ObjectIDs[$OldID];
				
				$ScriptContent = str_replace($OldID, $NewID, $ScriptContent);  
							
				$Log_UpdateScriptLinkID .= ("Script $ScriptID enthält Link OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
				}	
			else
				{
				$Log_UpdateScriptLinkID .= ("ERROR: Script $ScriptID enthält Link OldID: $OldID -> nicht in Referenz-ObjectIDs vorhanden -> Kontrolle erforderlich !!!\r\n");
				}
			}  
		IPS_SetScriptContent($ScriptID, $ScriptContent);
		}
	else
		{
		$Log_UpdateScriptLinkID .= ("Script $ScriptID enthält keine IDs\r\n");
		}	
	}

/*//////////////////////////////////////////////////////////////////////////////
Update der in Events enthaltenen LinkID`s (Eventscript und Triggervariable)
/*//////////////////////////////////////////////////////////////////////////////

function UpdateEventLinkID($EventID, $ObjectIDs, $EventConfigurationBuffer)
	{
	global $Log_UpdateEventLinkID; 

	$EventConfiguration = $EventConfigurationBuffer[$EventID];
	
	// ausgelöste Events nach Trigger-ObjectID durchsuchen und ersetzen ////////
	if(($EventConfiguration["EventType"] == 0) && !empty($EventConfiguration["TriggerVariableID"]))
		{
		$OldID = $EventConfiguration["TriggerVariableID"];
		
		if(isset($ObjectIDs[$OldID]))
			{
			$NewID = $ObjectIDs[$OldID];
			// Trigger-ObjectID setzen
			IPS_SetEventTrigger($EventID, $EventConfiguration["TriggerType"], $NewID);
			$Log_UpdateEventLinkID .= ("Event $EventID enthält Trigger-ObjectID OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
			}
		else
			{
			$Log_UpdateEventLinkID .= ("Error: Event $EventID enthält Trigger-ObjectID OldID: $OldID -> nicht in Referenz-ObjectIDs vorhanden -> Kontrolle erforderlich !!!\r\n");
			}
		}
	else
		{
		$Log_UpdateEventLinkID .= ("Event $EventID enthält keinen Link zu einer Trigger-ObjectID\r\n");
		}
		
	// Events nach EventScript-ObjectID durchsuchen und ersetzen ///////////////
	
	// Eventscript und eine ObjectID (5 Zahlen) sind vorhanden
	if(!empty($EventConfiguration["EventScript"]) && (preg_match("/^[0-9]{5}$/", $EventConfiguration["EventScript"])))
		{
		$OldID = $EventConfiguration["EventScript"];
		
		if(isset($ObjectIDs[$OldID]))
			{
			$NewID = $ObjectIDs[$OldID];
			// Eventscript-ObjectID setzen
			IPS_SetEventScript ($EventID, $NewID);
			$Log_UpdateEventLinkID .= ("Event $EventID enthält Eventscript-ObjectID OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
			}
		else
			{
			$Log_UpdateEventLinkID .= ("Error: Event $EventID enthält Eventscript-ObjectID OldID: $OldID -> nicht in Referenz-ObjectIDs vorhanden -> Kontrolle erforderlich !!!\r\n");
			}
		}
	else
		{
		$Log_UpdateEventLinkID .= ("Event $EventID enthält keinen Link zu einer Eventscript-ObjectID\r\n");
		}
	}

/*//////////////////////////////////////////////////////////////////////////////
Update der in Links enthaltenen TargetID`s
/*//////////////////////////////////////////////////////////////////////////////

function UpdateLinkTargetID($LinkID, $ObjectIDs, $LinkConfigurationBuffer)
	{
	global $Log_UpdateLinkTargetID;

	// alte LinkID
	$OldID = $LinkConfigurationBuffer[$LinkID]["TargetID"];
					
	if(isset($ObjectIDs[$OldID]))
		{
		$NewID = $ObjectIDs[$OldID];
		IPS_SetLinkTargetID($LinkID, $NewID);
		$Log_UpdateLinkTargetID .= ("Link $LinkID enthält TargetID OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
		}
	else
		{
		$Log_UpdateLinkTargetID .= ("Error: Link $LinkID enthält Link OldID: $OldID -> nicht in Referenz-ObjectIDs vorhanden -> Kontrolle erforderlich !!!\r\n");
		}
	}

############################# WFC ##############################################

/*//////////////////////////////////////////////////////////////////////////////
existente IDs im WFC auslesen
/*//////////////////////////////////////////////////////////////////////////////

function WFC_Existing_IDs($WFC_Existing_Items)
	{	
	foreach ($WFC_Existing_Items as $WFC_Existing_Item)
		{
		$WFC_Existing_IDs[] = $WFC_Existing_Item['ID'];
		}
	return $WFC_Existing_IDs;	
	}

/*//////////////////////////////////////////////////////////////////////////////
zu importierende IDs auf Vorhandensein vergleichen
/*//////////////////////////////////////////////////////////////////////////////

function WFC_Evaluate_IDs($WFC_Import_IDs, $WFC_Existing_IDs)
	{	
	$WFC_Evaluated_IDs = array();
	//$WFC_Evaluated_IDs = array('ID' => "", 'ID_Copy' => "");
	
	foreach ($WFC_Import_IDs as $WFC_Import_ID)
		{
		//Vergleich Original ID
		if(in_array($WFC_Import_ID, $WFC_Existing_IDs))
			{
			$WFC_Evaluated_IDs['ID'][] = $WFC_Import_ID;
			}
		//Vergleich ID_Copy
		if(in_array($WFC_Import_ID."_Copy", $WFC_Existing_IDs))
			{
			$WFC_Evaluated_IDs['ID_Copy'][] = $WFC_Import_ID."_Copy";
			}
		}
	return $WFC_Evaluated_IDs;	
	}
			
/*//////////////////////////////////////////////////////////////////////////////
Grundfunktion WFC-Baum aufbauen
/*//////////////////////////////////////////////////////////////////////////////

function WFC_Build_Tree($WFC_ID, $WFC_Tree, $WFC_Evaluated_IDs, $WFC_Status)
	{
	global $Log;
	
	// Baumelemente auslesen
	foreach($WFC_Tree as $ItemID => $Item)
		{	
		//echo("\r\nItem: "); print_r($Item);
		
		WFC_Set_Item($WFC_ID, $Item, $WFC_Evaluated_IDs, $WFC_Status);
		
		// auf Childs prüfen
		if(isset($Item['Childs']))
			{
			$Childs = $Item['Childs'];

			//echo("\r\nChilds: "); print_r($Childs);
			
			foreach($Childs as $ChildID => $Child)
				{
				//echo("\r\nChildID: "); print_r($ChildID);
				//echo("\r\nChild: "); print_r($Child);
				
				WFC_Build_Tree($WFC_ID, array($ChildID => $Child), $WFC_Evaluated_IDs, $WFC_Status);
				}
			}
		}
	IPS_ApplyChanges($WFC_ID);
	}

/*//////////////////////////////////////////////////////////////////////////////				
WFC_Item erstellen
/*//////////////////////////////////////////////////////////////////////////////

function WFC_Set_Item($WFC_ID, $WFC_Item, $WFC_Evaluated_IDs, $WFC_Status)
	{
	global $Log_WFC;

	//nicht zu modifizieren
	$ClassName = $WFC_Item['ClassName'];
	$Configuration = $WFC_Item['Configuration'];					
	$Position = $WFC_Item['Position'];
	$Visible = $WFC_Item['Visible'];
		
	switch($WFC_Status)
		{		
		case "0": //als WFC-Item anlegen
			
			$Log_WFC .= "WFC\t".$WFC_Item["ID"]." [ParentID: ".$WFC_Item["ParentID"]."] existiert noch nicht...\r\n";
			
			$ID       = $WFC_Item['ID'];
			$ParentID = $WFC_Item['ParentID'];
			
			$return = @WFC_AddItem($WFC_ID, $ID, $ClassName, $Configuration, $ParentID);
			
			//Fehlerabfrage
			if($return == true) {$Log_WFC .= "WFC\t".$ID." [ParentID: ".$ParentID."] neu angelegt\r\n";}
			else                {$Log_WFC .= "Fehler beim Anlegen von WFC_Item ".$ID." | ParentID: ".$ParentID."\r\n";}	
	
		break;
	
		case "1": //als WFC-Item_Copy anlegen
			
			$Log_WFC .= "WFC\t".$WFC_Item["ID"]." [ParentID: ".$WFC_Item["ParentID"]."] existiert...\r\n";
			
			$ID       = $WFC_Item['ID']."_Copy";
	
			//für Ursprung des Baumes [ParentID] = roottp setzen		
			if($WFC_Item['ParentID'] == "roottp") {$ParentID = "roottp";}
			else                                  {$ParentID = $WFC_Item['ParentID']."_Copy";}
			
			$return = @WFC_AddItem($WFC_ID, $ID, $ClassName, $Configuration, $ParentID);
			
			//Fehlerabfrage
			if($return == true) {$Log_WFC .= "WFC\t".$ID." -> als WFC_Item ".$ID." | ParentID: ".$ParentID." neu angelegt\r\n";}
			else                {$Log_WFC .= "Fehler beim Anlegen von WFC_Item ".$ID." | ParentID: ".$ParentID." als Copy\r\n";}				
	
		break;
	
		case "2": //als WFC-Item_Copy anlegen	
		
			$Log_WFC .= "WFC\t".$WFC_Item["ID"]." [ParentID: ".$WFC_Item["ParentID"]."] existiert bereits als Copy...\r\n";
			
			$ID       = $WFC_Item['ID']."_Copy";
			$ParentID = $WFC_Item['ParentID']."_Copy";
			
			//für Ursprung des Baumes [ParentID] = roottp setzen
			if($WFC_Item['ParentID'] == "roottp") {$ParentID = "roottp";}
			
			$return = @WFC_UpdateConfiguration($WFC_ID, $ID, $Configuration);
			
			//Fehlerabfrage
			if($return == true) {$Log_WFC .= "WFC\t".$ID." -> Update der Configuration\r\n";}
			else                {$Log_WFC .= "Fehler beim Update der Configuration von WFC_Item ".$ID."\r\n";}		
				
			$return = @WFC_UpdateParentID($WFC_ID, $ID, $ParentID);
			
			//Fehlerabfrage
			if($return == true) {$Log_WFC .= "WFC\t".$ID." -> Update der ParentID\r\n";}
			else                {$Log_WFC .= "Fehler beim Update der ParentID von WFC_Item ".$ID."\r\n";}	
	
		break;
		}
		
	$return = @WFC_UpdatePosition($WFC_ID , $ID, $Position);	
	
	//Fehlerabfrage
	if($return == true) {$Log_WFC .= "WFC\t".$ID." -> Update der Position\r\n";}
	else                {$Log_WFC .= "Fehler beim Update der Position von WFC_Item ".$ID."\r\n";}	
		
	$return = @WFC_UpdateVisibility($WFC_ID , $ID, $Visible);
	
	//Fehlerabfrage
	if($return == true) {$Log_WFC .= "WFC\t".$ID." -> Update der Visibility\r\n";}
	else                {$Log_WFC .= "Fehler beim Update der Visibility von WFC_Item ".$ID."\r\n";}		
	}	

/*//////////////////////////////////////////////////////////////////////////////
Update der in Events enthaltenen ScriptLinkID`s
/*//////////////////////////////////////////////////////////////////////////////

function UpdateWFCLinkID($WFC_ID, $Item, $ObjectIDs)
	{
	global $Log_UpdateWFCLinkID; 

	$Configuration = json_decode($Item["Configuration"], true);
		
	$ID = $Item["ID"];	
		
	//WFC_Items nach Verlinkung zu einer ObjectID durchsuchen und ersetzen//////
	if(isset($Configuration["baseID"]))
		{
		$OldID = $Configuration["baseID"];
		//echo("\r\nOldID: ".$OldID);
		$NewID = $ObjectIDs[$OldID];
		//echo("\r\nNewID: ".$NewID);
		$Configuration["baseID"] = $NewID;
		$Configuration = json_encode($Configuration); //print_r($Configuration);
	
		WFC_UpdateConfiguration($WFC_ID, $ID, $Configuration);
		IPS_ApplyChanges($WFC_ID);
		$Log_UpdateWFCLinkID .= ("WFC-Item mit ID: $ID enthält OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
		}
	else
		{
		$Log_UpdateWFCLinkID .= ("WFC-Item mit ID: $ID enthält keinen Link zu einem Object\r\n");
		}
	}	
			
/*//////////////////////////////////////////////////////////////////////////////
Dateipfade erstellen
/*//////////////////////////////////////////////////////////////////////////////

function CreateDirectoryPath($Path)
	{
	if (file_exists($Path))
		{
		return true;
		}
	$SubDirectoryPath = substr($Path, 0, strrpos($Path, "/", -2) + 1 );
	
	if(CreateDirectoryPath($SubDirectoryPath) && is_writable($SubDirectoryPath))
		{
		return mkdir($Path);
		}
	
	return false;
	}

/*//////////////////////////////////////////////////////////////////////////////
Update der in WebFront - Scripten enthaltenen ID`s
/*//////////////////////////////////////////////////////////////////////////////

function UpdateWebFrontScriptLinkID($File_old, $ObjectIDs, $FileName)
	{
	global $Log;
		
	/*//////////////////////////////////////////////////////////////////////////
	suche 5stellige Zahlen in Dateien mit Endung .php | .txt
	----------------------------------------------------------------------------
	Treffer nur wenn
	- davor: keine Zahl \d | kein : | kein Groß- oder Kleinbuchstabe
	- 5 Zahlen
	- danach: keine Zahl 
	/*//////////////////////////////////////////////////////////////////////////
	
	preg_match_all ("/(?<![\d\:A-Za-z])([1-5][0-9]{4})(?!\d)/", $File_old, $OldIDs); 
	
	if(!empty($OldIDs[0]))
		{			
		foreach($OldIDs[0] as $OldID)
			{
			if(in_array($OldID, $ObjectIDs))
				{
				//echo("OldID: $OldID\r\n");
				$NewID = $ObjectIDs[$OldID];
				
				$File_new = str_replace($OldID, $NewID, $File_old);		
	
				$Log .= ("Datei $FileName enthält OldID: $OldID -> ersetze durch NewID: $NewID\r\n");
				}	
			else
				{
				$File_new = $File_old;
				
				$Log .= ("ERROR: Datei $FileName enthält Link OldID: $OldID -> nicht in Referenz-ObjectIDs vorhanden\r\nOriginaldatei geschrieben -> Kontrolle erforderlich !!!\r\n");
				}
			}
		}
	else
		{
		$File_new = $File_old;
		
		$Log .= ("Datei $FileName enthält keine IDs\r\n");
		}	
	
	return $File_new;
	}	

/*//////////////////////////////////////////////////////////////////////////////
WFC_Tree auslesen
/*//////////////////////////////////////////////////////////////////////////////

function WFC_Read_Tree($WFC_NewID, $WFC_Export_Tree, $ObjectIDs)
	{
	global $Log_WFC_Items;

	foreach($WFC_Export_Tree as $ItemID => $Item)
		{	
		UpdateWFCLinkID($WFC_NewID, $Item, $ObjectIDs);
		
		// auf Childs prüfen
		if(isset($Item["Childs"]))
			{
			$Childs = $Item["Childs"];
			
			foreach($Childs as $ChildID => $Child)
				{
				WFC_Read_Tree($WFC_NewID, array($ChildID => $Child), $ObjectIDs);
				}
			}
		}
	}

/*//////////////////////////////////////////////////////////////////////////////
Temporäre globale Variablen löschen
/*//////////////////////////////////////////////////////////////////////////////

unset( $MC_InstanceConfig );	
unset( $MC_CustomVarProfiles );
unset( $MC_ScriptContents );
unset( $MC_MediaContents );
unset( $Exist );

unset( $ObjectIDs );
unset( $CategoryIDs );
unset( $InstanceIDs );
unset( $VariableIDs );
unset( $ScriptIDs );
unset( $EventIDs );
unset( $MediaIDs );
unset( $LinkIDs );

unset( $Log_SetObject );

unset( $InstanceConnectBuffer );
unset( $InstanceConfigurationBuffer );
unset( $VariableConfigurationBuffer );
unset( $EventConfigurationBuffer );
unset( $LinkConfigurationBuffer );

unset( $Log_UpdateInstanceConnectionID );
unset( $Log_UpdateInstanceConfiguration );
unset( $Log_UpdateVariableCustomActionLinkID );
unset( $Log_UpdateScriptLinkID );	
unset( $Log_UpdateEventLinkID ); 
unset( $Log_UpdateLinkTargetID );
	
?>